# =============================================================================
# IRC2WWW Gateway v1.0 by Ernst <baschneo@trick.informatik.uni-stuttgart.de>
#
# *ADVANCED version*
# =============================================================================
#
# DO *NOT* INSTALL WITHOUT READING:
#
# This script will allow your bot to act as an "gateway" between IRC and a
# WWW browser.  This means, users will be able the "chat" throught your bot
# using just a WWW browser (e.g.  Netscape, Explorer...).  It requires an
# FRAMES compatible browser, and permits to watch the channel activity in one
# frame, while sending text to the channel (the bot will then speak it in the
# real IRC).
#
# Nothing (really!) is required for this to work, except the working
# _and patched_ 1.1.5 eggdrop (patched with eggdrop1.1.5-eb1.patch).  There is
# no need of an HTTP server, access to CGI, nothing at all.  With this script,
# your eggdrop will be the HTTP server.
#
# It is an extremely complex script, and you probably don't want to use your
# "production" bot with it.  If you can and want, create another bot
# exclusively for using it.  This is also wise, since this script is much more
# sensitive to bugs.  It explores eggdrops control/listen tcl commands, which
# weren't explored that much before, thus "strange" and "unexpected"
# behaviours might be encountered (bot crashing).
#
# I have once again heavilly commented the script, so you can not only have	fun
# with it, but maybe learn something.  TCL is easy and much more powerful than
# one might expect.
#
# -----------------------------------------------------------------------------
# There are two versions for eggdrop 1.1.5 of this script.
#
#                   THIS IS THE *ADVANCED* IRC2WWW SCRIPT
# 
# It requires you to recompile your 1.1.5 eggdrop bot with the
# eggdrop1.1.5-eb1.patch patch applied! This is *vital* and if you don't want
# or can do it, you must use the *STANDARD* version of this script.
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# INSTALLATION
# -----------------------------------------------------------------------------
# 1) apply the eggdrop1.1.5-eb1.patch to the bot and recompile it
# 2) configure the CONFIGURATION part below to your likings
# 3) add the script to your bots config
# 4) .rehash or .restart and watch the log if it says "...Serving on port XXX"
# 5) start your FRAMES compatible browser (e.g. Netscape) and enter the URL:
#      http://your.bots.hostname:port/
#    'your.bots.hostname' is the host address where your bot sits
#    'port' is the wwwport you defined in the CONFIGURATION
# 6) have fun! :)

# -----------------------------------------------------------------------------
# CONFIGURATION
# -----------------------------------------------------------------------------

# Channel for which you want to enable the www-irc gateway
set wwwchan "#brasil"

# Port to listen for HTTP requests
set wwwport 8080

# Maximum allowed www chatters.  Set to 0 for no limit
set maxchatters 4

# Which flag is needed on bot to enter www-system? ('-' for none)
set wwwflag -

# Strip HTML codes from IRC text, before sending to WEB?
set wwwnohtml 0

# Attempt to translate mIRC colors to HTML colors? (no bg-colors supported)
set wwwmirccolors 1

# Debug level.
# Valid values are: 0 (no logging), 1 (normal) and 2 (development debugging)
set wwwdebug 1

# Update of socket writes for streaming (queued) channel data (in secs)
set sockupdate 1

# -----------------------------------------------------------------------------
# THE SCRIPT BEGINS...
# -----------------------------------------------------------------------------

set wwwchan [string tolower $wwwchan]
set wwwver "0.9adv"

proc www_debug { level text } {
	global wwwdebug
	if {$level <= $wwwdebug} { putlog "www2irc: $text" }
}

# Little proc to compare string ignoring case, used in sorting
proc icompare {arg1 arg2} { return [string compare [string tolower $arg1] [string tolower $arg2]] }

# == SOCKET PROCEDURES =======================================================
# Everything used to be "sockets" or "channels" when I did this for egg 1.0.
# Now with 1.1 we have control/listen, and each connected socket gets an
# unique index, called "idx", but you will still see a lot the word "socket":
# they all mean "idx".
#
# This version of irc2www.tcl will use the "new" control standard which is
# added by eggdrop1.1.5-eb1.patch. If you have NOT applied this patch to your
# eggdrop 1.1.5, this script will *not* work. Use the "standard" version of
# irc2www instead.

# Receives a new socket connection, sets initial vars and get ready for I/O
proc new_idx {newidx} {
	global sockserv sockmode sockpresent sockdata sockresponse
	www_debug 2 "new connection: [dcchost $newidx] (idx: $newidx)"
	# Just in case something got left from older connections
	cleansockinfo $newidx
	# No input or output yet
	set sockmode(i!$newidx) "NO"
	set sockmode(o!$newidx) "NO"
	set sockresponse($newidx) "200 OK"
	# Now input will go to the "readsock" proc
	control $newidx "readsock"
	return 0
}

# Main routine which gets input from the WWW-browser, parses the requests,
# and does anything which might be needed.
# 1) reads all data coming from browser (header)
# 2) sends data back using writesock according to request
# 3) if data is a single page, instructs writesock to CLOSE after sending.
#    if data is the chat frame, instructs writesock to keep streaming data.
proc readsock {idx input} {
	global sockserv sockmode sockdata sockqueue maxchatters wwwchan server wwwver
	global userin getstring botnick	wwwflag sockresponse
	if {$input == "**EOF**" || ![info exists sockmode(o!$idx)]} {
		# This is the end...
		sockclose $idx
		return 1
	}
	if {$input == "" && $sockmode(o!$idx) == "NO" && $sockmode(i!$idx) != "NO"} {
		# HTTP header completely received
		set sockmode(o!$idx) "INIT"
		# Go ahead, initialize the HTTP response
		writesock $idx
		return 0
	}
	if {$sockmode(i!$idx) == "GOTPOST"} {
		if {[string length $input] > 0 && ![string match "*:*" $input]} {
			# Got Content of POST request
			set getstring($idx) [concat $getstring($idx) [parse_get $input]]
			# Threat things as if a GET with parameters was send
			set sockmode(i!$idx) "GOTGET"
		} {
			# Does nothing until the Content is received
			return 0
		}
	}
	# First input line must be the REQUEST (as of HTTP specs)
	if {$sockmode(o!$idx) == "NO" && $sockmode(i!$idx) == "NO"} {
		if {[string match "GET *" [string toupper $input]]} {
			# This parses the GET string into a tcl-list
			set getstring($idx) [parse_get $input]
			www_debug 2 "Requested GET: [lindex $getstring($idx) 0]"
			set sockmode(i!$idx) "GOTGET"
		} elseif {[string match "POST *" [string toupper $input]]} {
			set getstring($idx) [parse_get $input]
			www_debug 2 "Requested POST: [lindex $getstring($idx) 0]"
			set sockmode(i!$idx) "GOTPOST"
			return 0
		} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			set sockresponse($idx) "501 Not implemented"
			www_debug 2 "Wrong access method: [lindex $input 0]"
			lappend sockqueue($idx) "<HTML><HEAD><TITLE>Method not implemented</TITLE></HEAD>"
			lappend sockqueue($idx) "<BODY><H1>501 Not implemented</H1>"
			lappend sockqueue($idx) "The request [string toupper [string range $input 0 [expr [string first " " $input] - 1]]] could not be processed.</P>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
	}
	if {$sockmode(i!$idx) == "GOTGET"} {
		switch -- [string tolower [lindex $getstring($idx) 0]] {
			"/" {
				set sockmode(i!$idx) "SENDANDCLOSE"
				www_debug 2 "User at entry door"
				lappend sockqueue($idx) "<HTML><HEAD><TITLE>IRC Gateway for $wwwchan</TITLE></HEAD>"
				lappend sockqueue($idx) "<BODY BGCOLOR=#ffffff>"
				lappend sockqueue($idx) "<FORM ACTION=\"/enter\" METHOD=POST>"
				lappend sockqueue($idx) "<H2>IRC Gateway for $wwwchan<HR NOSHADE SIZE=1></H2>"
				lappend sockqueue($idx) "<P>Please enter your login name to enter the IRC idx.</P>"
				lappend sockqueue($idx) "<UL><TABLE CELLPADDING=0 CELLSPACING=0>"
				lappend sockqueue($idx) "<TR><TD>User: </TD><TD><INPUT TYPE=text NAME=user SIZE=25></TD></TR>"
				lappend sockqueue($idx) "<TR><TD>Pass: </TD><TD><INPUT TYPE=password NAME=pass SIZE=25></TD></TR>"
				lappend sockqueue($idx) "<TR><TD></TD><TD><INPUT TYPE=submit VALUE=Login></TD></TR>"
				lappend sockqueue($idx) "</TABLE></UL>"
				lappend sockqueue($idx) "<P><HR NOSHADE SIZE=1><FONT SIZE=-1>via eggdrop with www2irc.tcl v$wwwver by Ernst</P>"
				lappend sockqueue($idx) "</BODY></HTML>"
				return 0
			}
			# Entry point, requested from entry form. Send FRAME struct
			"/enter" {
				foreach thisarg [lrange $getstring($idx) 1 end] {
					if {[lindex $thisarg 0] == "user"} { set user [lindex $thisarg 1] }
					if {[lindex $thisarg 0] == "pass"} { set pass [lindex $thisarg 1] }
				}
			}
			# Lower window in frame, for sending text to irc
			"/talk" {
				set id 0
				foreach thisarg [lrange $getstring($idx) 1 end] {
					if {[lindex $thisarg 0] == "id"} { set id [lindex $thisarg 1] }
					if {[lindex $thisarg 0] == "chat"} { set chat [lindex $thisarg 1] }
					if {[lindex $thisarg 0] == "method"} { set method [lindex $thisarg 1] }
				}
				if {![info exists userin(user!$id)]} {
					set sockmode(i!$idx) "SENDANDCLOSE"
					www_debug 2 "ERROR: Wrong ID for channel"
					lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
					lappend sockqueue($idx) "<H2>Please enter the system <A HREF=\"/\">from the main entry</A>.</H2>"
					lappend sockqueue($idx) "</BODY></HTML>"
					return 0
				}
				set sockmode(i!$idx) "SENDANDCLOSE"
				lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#95d095><CENTER><FORM ACTION=\"/talk\" METHOD=GET>"
				lappend sockqueue($idx) "<TABLE><TR><TD></TD><TD>Text:</TD><TD></TD></TR>"
				lappend sockqueue($idx) "<TR><TD><SELECT NAME=method><OPTION VALUE=say SELECTED>say"
				lappend sockqueue($idx) "<OPTION VALUE=me>action</SELECT></TD>"
				lappend sockqueue($idx) "<TD><INPUT TYPE=text NAME=chat SIZE=40></TD>"
				lappend sockqueue($idx) "<TD><INPUT TYPE=submit VALUE=Send to channel></TD></TR>"
				lappend sockqueue($idx) "</TABLE><INPUT TYPE=hidden NAME=id VALUE=$id></FORM></CENTER></BODY></HTML>"
				if {[info exist chat]} {
					if {$method == "say"} {
						# Simulate a pubm, so that other www-chatters get the message
						sock_pubm "$userin(user!$id)" "$botnick" "$id" "$wwwchan" "[url2human $chat]"
						putserv "PRIVMSG $wwwchan :\[WWW\] <$userin(user!$id)> [url2human $chat]"
					} {
						# Simulate an action, so that other www-chatters get the message
						sock_act "$userin(user!$id)" "$botnick" "$id" "$wwwchan" "ACTION" "[url2human $chat]"
						putserv "PRIVMSG $wwwchan :\001ACTION \[WWW\] * $userin(user!$id) [url2human $chat]\001"
					}
				}
				return 0
			}
			# Upper frame, containing the irc channel stream
			"/channel" {
				set id 0
				foreach thisarg [lrange $getstring($idx) 1 end] {
					if {[lindex $thisarg 0] == "id"} { set id [lindex $thisarg 1] }
				}
				if {![info exists userin(user!$id)]} {
					set sockmode(i!$idx) "SENDANDCLOSE"
					www_debug 2 "ERROR: Wrong ID for channel"
					lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
					lappend sockqueue($idx) "<H2>Please reenter the channel from <A HREF=\"/\">the beginning</A>.</H2>"
					lappend sockqueue($idx) "</BODY></HTML>"
					return 0
				}
				set userin(talk!$id) $idx
				loginuser $idx $id
				return 0
			}
			# Nothing, send something to make requester happy
			default {
				set sockmode(i!$idx) "SENDANDCLOSE"
				set sockresponse($idx) "404 Not Found"
				www_debug 2 "Bad path: [string tolower [lindex $getstring($idx) 0]]"
				lappend sockqueue($idx) "<HTML><HEAD><TITLE>File Not found</TITLE></HEAD>"
				lappend sockqueue($idx) "<BODY><H1>404 File not found</H1>"
				lappend sockqueue($idx) "The requested URL [string tolower [lindex $getstring($idx) 0]] was not found on this server.</P>"
				lappend sockqueue($idx) "</BODY></HTML>"
				return 0
			}
		}
		# In case of /enter, proceed here
		if {![info exist user] || ![info exist pass]} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			www_debug 2 "ERROR: No user/pass entered"
			lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
			lappend sockqueue($idx) "<H2>No User or Password entered. <A HREF=\"/\">Try again</A>.</H2>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
		if {$maxchatters != 0 && [llength [wwwusers]] >= $maxchatters} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			www_debug 1 "ERROR: Max number of chatters reached"
			lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
			lappend sockqueue($idx) "<H2>Too many chatters already. <A HREF=\"/\">Try again</A>.</H2>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
		if {![validuser $user] || ![passwdok $user $pass]} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			www_debug 2 "ERROR: Wrong user/password"
			lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
			lappend sockqueue($idx) "<H2>No such user, or password incorrect. <A HREF=\"/\">Try again</A>.</H2>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
		if {![matchattr $user $wwwflag] && $wwwflag != "-"} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			www_debug 2 "ERROR: Only flag $wwwflag allowed"
			lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
			lappend sockqueue($idx) "<H2>Only avaliable for users with flag <STRONG>$wwwflag</STRONG>.</H2>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
		if {![onchan $botnick $wwwchan]} {
			set sockmode(i!$idx) "SENDANDCLOSE"
			www_debug 1 "ERROR: I am not in $wwwchan :("
			lappend sockqueue($idx) "<HTML><BODY BGCOLOR=#ffffff>"
			lappend sockqueue($idx) "<H2>The irc2www gateway is currently offline, sorry! :(</H2>"
			lappend sockqueue($idx) "</BODY></HTML>"
			return 0
		}
		# Generates an user ID for this connection
		set userid [string range [unixtime] 3 end]
		www_debug 1 "Connected: $user ID: $userid"
		set userin(user!$userid) $user
		set userin(talk!$userid) 0
		set sockmode(i!$idx) "SENDANDCLOSE"
		lappend sockqueue($idx) "<HTML><HEAD><TITLE>Canal $wwwchan</TITLE></HEAD>"
		lappend sockqueue($idx) "<FRAMESET FRAMEBORDER=0 FRAMESPACING=0 BORDER=0 ROWS=\"*,100\">"
		lappend sockqueue($idx) "<FRAME SCROLLING=\"AUTO\" NAME=\"banner\" SCROLLING=\"AUTO\" NAME=\"channel\" SRC=\"/channel?id=$userid\" NORESIZE>"
		lappend sockqueue($idx) "<FRAME MARGINWIDTH=7 MARGINHEIGHT=7 SCROLLING=\"NO\" NAME=\"talk\" SRC=\"/talk?id=$userid\" NORESIZE>"
		lappend sockqueue($idx) "</FRAMESET><NOFRAMES>"
		lappend sockqueue($idx) "<BODY BGCOLOR=#ffffff TEXT=#000000 LINK=#006600 VLINK=#ff0000>"
		lappend sockqueue($idx) "<H1>Sorry! You need frames!</H1>"
		lappend sockqueue($idx) "</BODY></HTML>"
		return 0
	}
	if {$sockmode(i!$idx) == "NEVERCLOSE" && [string match "User-Agent:*" $input]} {
		# Capture the browser name in the "/channel" frame request
		set id [sock2id $idx]
		if {$id != ""} {
			set userin(agent!$id) [string range $input 12 end]
			regsub -all "Mozilla" $userin(agent!$id) "Netscape" userin(agent!$id)
			www_debug 2 "$userin(user!$id) is using $userin(agent!$id)"
		}
		return 0
	}
}

# User logged in correctly, sends the beginning of the channel page, which
# contains current topic and users.  sockmode is set to NEVERCLOSE.
proc loginuser {channel id} {
	global sockmode sockqueue wwwchan userin server
	set sockmode(i!$channel) "NEVERCLOSE"
	lappend sockqueue($channel) "<HTML><HEAD><TITLE>$wwwchan: [topic $wwwchan]</TITLE></HEAD>"
	lappend sockqueue($channel) "<BODY BGCOLOR=#ffffff>"
	lappend sockqueue($channel) "<H2>Welcome to $wwwchan, $userin(user!$id)<HR></H2>"
	lappend sockqueue($channel) "<P>I am currently on <I>[lindex [split $server :] 0]</I>. Have fun!"
	if {[topic $wwwchan] != ""} {
		lappend sockqueue($channel) "<BR>The current topic is <B><TT>[topic $wwwchan]</TT></B>.</P>"
	}
	lappend sockqueue($channel) "<PRE>"
	set theops ""
	set theusers ""
	set thewww ""
	foreach nick [chanlist $wwwchan] {
		if {[isop $nick $wwwchan]} {
			lappend theops $nick
		} {
			lappend theusers $nick
		}
	}
	foreach wwwuser [wwwusers] {
		lappend thewww $userin(user![sock2id $wwwuser])
	}
	set theops [lsort -command icompare $theops]
	set theusers [lsort -command icompare $theusers]
	set thewww [lsort -command icompare $thewww]
	regsub -all " " $theops ", " theops
	regsub -all " " $theusers ", " theusers
	regsub -all " " $thewww ", " thewww
	# Make lists of strings at the proper size to fit the page
	set theops [wordwrap $theops [expr 70]]
	set theusers [wordwrap $theusers [expr 70]]
	set thewww [wordwrap $thewww [expr 70]]
	if {[llength $theops] > 0} {
		lappend sockqueue($channel) "<B>      Ops:</B> [lindex $theops 0]"
		if {[llength $theops] > 1} {
			foreach nextops [lrange $theops 1 end] {
				lappend sockqueue($channel) "       $nextops"
			}
		}
	}
	if {[llength $theusers] > 0} {
		lappend sockqueue($channel) "<B>    Users:</B> [lindex $theusers 0]"
		if {[llength $theusers] > 1} {
			foreach nextusers [lrange $theusers 1 end] {
				lappend sockqueue($channel) "       $nextusers"
			}
		}
	}
	if {[llength $thewww] > 0} {
		lappend sockqueue($channel) "<B>WWW Users:</B> [lindex $thewww 0]"
		if {[llength $thewww] > 1} {
			foreach nextwww [lrange $thewww 1 end] {
				lappend sockqueue($channel) "       $nextwww"
			}
		}
	}
	lappend sockqueue($channel) "</PRE><HR>"
}

# Closes a socket, removing associated variables
proc sockclose {idx} {
	global sockdata sockmode wwwchan userin botnick
	www_debug 2 "-- CLOSING $idx --"
	if {[lsearch [wwwusers] $idx] != -1} {
		set id [sock2id $idx]
		if {$id != ""} {
			sock_sign "$userin(user!$id)" "$botnick" "$id" "$wwwchan" "WWW Connection Closed"
			putserv "NOTICE $wwwchan :\0033*** $userin(user!$id) Quits $wwwchan: WWW Connection Closed\003"
		}
	}
	cleansockinfo $idx
	killdcc $idx
	return 0
}

# Cleans up all mess created with any opened channel. 
# Also cleans user information (bound to IDs) when it was a "channel" frame
# which was closed (assumes that chatter is no longer there)
proc cleansockinfo {channel} {
	global sockqueue sockdata sockmode getstring userin
	catch {unset getstring($channel)}
	catch {unset sockqueue($channel)}
	catch {unset sockmode(i!$channel)}
	catch {unset sockmode(o!$channel)}
	set id [sock2id $channel]
	if {$id != ""} {
		catch {unset userin(join!$id)}
		catch {unset userin(talk!$id)}
		catch {unset userin(user!$id)}
		catch {unset userin(agent!$id)}
	}
	return 0
}

# Checks if there is something to write to any given socket.
# Writes data to socked if there is something in queue, closes it if wanted
proc writesock {channel} {
	global sockmode sockqueue sockupdate sockdata userin wwwchan botnick
	global wwwver version sockresponse
	if {![info exists sockmode(o!$channel)]} { return 0 }
	if {$sockmode(o!$channel) == "INIT"} {
		www_debug 2 "-- BEGIN HTML ($channel) --"
		# HTTP 1.0 - much easier, no bizarre Keep-alives
		putdcc $channel "HTTP/1.0 $sockresponse($channel)"
		putdcc $channel "Server: irc2www.tcl/$wwwver eggdrop/[lindex $version 0]"
		putdcc $channel "Content-Type: text/html"
		putdcc $channel ""
		set sockmode(o!$channel) "OK"
		writesock $channel
		return 0
	}
	if {$sockmode(o!$channel) == "OK"} {
		# Nothing queued, abort
		if {![info exists sockqueue($channel)]} {
			utimer $sockupdate "writesock $channel"
			return 0
		}
		foreach nextitem $sockqueue($channel) {
			putdcc $channel "$nextitem"
		}
		# Too late, already closed...
		if {![info exists sockmode(i!$channel)]} { return 0 }
		unset sockqueue($channel)
		set id [sock2id $channel]
		if {$sockmode(i!$channel) == "NEVERCLOSE" && $id != ""} {
			if {![info exist userin(join!$id)]} {
				set userin(join!$id) 1
				sock_join "$userin(user!$id)" "$botnick" "$id" "$wwwchan"
				if {$userin(agent!$id) == ""} {
					putserv "NOTICE $wwwchan :\0033*** $userin(user!$id) WWW-joins $wwwchan\003"
				} {
					putserv "NOTICE $wwwchan :\0033*** $userin(user!$id) WWW-joins $wwwchan \(using $userin(agent!$id)\)\003"
				}
			}
			utimer $sockupdate "writesock $channel"
		}
		if {$sockmode(i!$channel) == "SENDANDCLOSE"} {
			sockclose $channel
			return 0
		}
	}
}


# == INFO PROCS ==============================================================

# Given an idx of a dcc connection, returns hostname, "" if no such connection
proc dcchost {idx} {
	foreach dccentry [dcclist] {
		if {[lindex $dccentry 0] == $idx} {
			return [lindex [split [lindex $dccentry 2] ":"] 1]
		}
	}
	return ""
}

# A listing of socket bound to the "/channel" frame. If the socket is still
# active, the chatter is still there.
proc wwwusers {} {
	global userin
	set chatters ""
	foreach	entry [array names userin] {
		if {[string match "user*" $entry]} {
			set id [string range $entry 5 end]
			if [info exists userin(talk!$id)] {
				lappend chatters $userin(talk!$id)
			}
		}
	}
	return $chatters
}

# Given a socket number of the "channel" frame, returns the ID of that user.
proc sock2id {channel} {
	global userin
	foreach	entry [array names userin] {
		if {[string match "talk*" $entry]} {
			if {$userin($entry) == $channel} {
				return "[string range $entry 5 end]"
			}
		}
	}
	return ""
}


# == CONVERSION/PARSING STUFF ================================================

# Parses a GET request with multiple inputs
# 1)  input is:  GET /enter?bla=xxx&bla2=yyy
# returns list:  {/enter} {bla xxx} {bla2 yyy}
# 2)  input is:  POST /enter         <- POST Request
#      returns:  /enter
# 3)  input is:  bla=xxx&bla2=yyy    <- POST Content
# returns list:  {bla xxx} {bla2 yyy}
# by Ernst
proc parse_get {input} {
	set mode 0
	if {![string match "GET *" [string toupper $input]] && \
	    ![string match "POST *" [string toupper $input]]} { set mode 1 }
	if {$mode == 0} {
		set input [string range $input [expr [string first " " $input] + 1] end]
	}
	set path ""
	set curitem ""
	set curvalue ""
	for {set i 0} {$i < [string length $input]} {incr i} {
		set thischar [string index $input $i]
		if {$mode == 0} {
			if {$thischar == "?" || $thischar == " "} {
				lappend output $path
				set path ""
				if {$thischar == " "} {
					set mode 2
				} {
					set mode 1
				}
				continue
			}
			set path $path$thischar
		}
		if {$mode == 1} {
			if {$thischar == "="} {
				set mode 2
				continue
			} {
				set curitem $curitem$thischar
			}
		}
		if {$mode == 2} {
			if {$thischar == "&" || $thischar == " "} {
				lappend output "$curitem $curvalue"
				set curitem ""
				set curvalue ""
				set mode 1
				if {$thischar == " "} { break }
				continue
			} {
				set curvalue $curvalue$thischar
			}
		}
	}
	if {$path != ""} { lappend output "$path" }
	if {$curitem != ""} { lappend output "$curitem $curvalue" }
	return $output
}

# Cleans text from HTML codes ('<' into '&lt;' and '>' into '&gt;')
proc htmlclean { arg } {
	global wwwnohtml
	if {$wwwnohtml} {
		regsub -all "<" $arg "\\\&lt;" arg
		regsub -all ">" $arg "\\\&gt;" arg
	}
	return $arg
}

# Converts a URL GET string into human readable stuff
# Stolen uncgi v2.1 from dl@hplyot.obspm.fr
proc url2human {buf} {
	# unescape chars escaped by httpd
	regsub -all {\\(.)} $buf {\1} buf ;
	# escape dangerous characters ([, $ and ")
	regsub -all {([[$"])} $buf {\\\1} buf;
	#translate + into space
	regsub -all {\+} $buf { } buf
	# everything is ready, now the real work :
	regsub -all -nocase {%([a-fA-F0-9][a-fA-F0-9])} $buf {[format %c 0x\1]} buf
	# And now lets replace all those escaped back, along with excuting of
	# the formats :
	eval return \"$buf\"
	# now everything is in buf, but translated, nice trick no ?
}

# Converts mIRC colours/bold/underline to HTML <FONT>s (where possible)
# by Ernst
proc mirc2html { arg } {
	global wwwmirccolors
	if {!$wwwmirccolors} { return $arg }
	if {[string length $arg] == 0} { return 0 }
	set colors {
		"ffffff" "000000" "001080" "009a00"	"ff0000" "9a0000" "a000a0" "ff7700"
		"ffff00" "00ff00" "50a0a0" "00ffff" "0000ff" "ff00ff" "888888" "d0d0d0"
	}
	set open(color) 0
	set open(bold) 0
	set open(underline) 0
	set output ""
	for {set i 0} {$i < [string length $arg]} {incr i} {
		if {[string index $arg $i] == "\003"} {
			set thiscolor ""
			if {$open(color) == 1} {
				if {[string match \[0-9\] [string index $arg [expr $i + 1]]]} {
					set output "$output</FONT>"
					set open(color) 0
				}
			}
			if {$open(color) == 0} {
				set open(color) 1
				incr i
				if {![string match \[0-9\] [string index $arg $i]]} {
					set open(color) 0
					set output "$output[string index $arg $i]"
				} {
					set thiscolor [string index $arg $i]
					incr i
					if {[string match \[0-9\] [string index $arg $i]]} {
						set thiscolor "$thiscolor[string index $arg $i]"
						incr i
					}
					set thiscolor [expr $thiscolor % 16]
					set output "$output<FONT COLOR=\"#[lindex $colors $thiscolor]\">[string index $arg $i]"
				}
			} {
				set open(color) 0
				set output "$output</FONT>"
			}
		} elseif {[string index $arg $i] == "\002"} {
			if {$open(bold) == 0} {
				set open(bold) 1
				set output "$output<B>"
			} {
				set open(bold) 0
				set output "$output</B>"
			}
		} elseif {[string index $arg $i] == "\031" || [string index $arg $i] == "\037"} {
			if {$open(underline) == 0} {
				set open(underline) 1
				set output "$output<U>"
			} {
				set open(underline) 0
				set output "$output</U>"
			}
		} {
			set output "$output[string index $arg $i]"
		}
	}
	if {$open(color) == 1} { set output "$output</FONT>" }
	if {$open(underline) == 1} { set output "$output</U>" }
	if {$open(bold) == 1} {	set output "$output</B>" }
	return $output
}

# Wordwrap returns a list where each element is a string with max 'len' width.
# Words from 'input' are split in lines on spaces (wordwrapping, you know...)
# by Ernst
proc wordwrap {input len} {
	set pos 1
	while {$pos > 0} {
		set pos [string last " " [string range $input 0 $len]]
		if {[string length $input] < $len} {
			lappend thelist "[string range $input 0 end]"
			set pos 0
		} {	lappend thelist "[string range $input 0 $pos]"	}
		set input "[string range $input [expr $pos + 1] end]"
	}
	return $thelist
}

# == BINDINGS ================================================================

bind pubm - "$wwwchan *" sock_pubm
bind join - "$wwwchan *" sock_join
bind part - "$wwwchan *" sock_part
bind sign - "$wwwchan *" sock_sign
bind kick - "$wwwchan *" sock_kick
bind nick - "$wwwchan *" sock_nick
bind mode - "$wwwchan *" sock_mode
bind splt - "$wwwchan *" sock_splt
bind rejn - "$wwwchan *" sock_rejn
bind topc - "$wwwchan *" sock_topc
bind ctcp - ACTION sock_act

# Binding on public chat or if any WWW-participant says something
proc sock_pubm { nick uhost hand chan arg } {
	global sockqueue botnick
	set pubm [mirc2html [htmlclean $arg]]
	if {$uhost == $botnick} {
		foreach thissock [wwwusers] {
			if {$hand == [sock2id $thissock]} {
				lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys COLOR=#ff0000>&lt;$nick&gt; </FONT></TD>"
				lappend sockqueue($thissock) "<TD><FONT FACE=FIXEDSYS>$pubm</FONT></TD></TR></TABLE>"
			} {
				lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys>&lt;$nick&gt; </FONT></TD>"
				lappend sockqueue($thissock) "<TD><FONT FACE=FIXEDSYS>$pubm</FONT></TD></TR></TABLE>"
			}
		}
	} {
		foreach thissock [wwwusers] {
			lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys>&lt;$nick&gt; </FONT></TD>"
			lappend sockqueue($thissock) "<TD><FONT FACE=FIXEDSYS>$pubm</FONT></TD></TR></TABLE>"
		}
	}
	return 0
}

# Binding on public actions or if any WWW-participant acts
proc sock_act { nick uhost hand dest keyword arg } {
	global sockqueue wwwchan botnick
	if {[string tolower $dest] == $wwwchan} {
		set pubact [mirc2html [htmlclean $arg]]
		if {$uhost == $botnick} {
			foreach thissock [wwwusers] {
				if {$hand == [sock2id $thissock]} {
					lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys COLOR=#aa4488>* </FONT></TD>"
					lappend sockqueue($thissock) "<TD><FONT FACE=FixedSys><FONT COLOR=#ff0000>$nick</FONT> <FONT COLOR=#aa4488>$pubact</FONT></TD></TR></TABLE>"
				} {
					lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys COLOR=#aa4488>* </FONT></TD>"
					lappend sockqueue($thissock) "<TD><FONT FACE=FixedSys><FONT COLOR=#aa4488>$nick $pubact</FONT></TD></TR></TABLE>"
				}
			}
		} {
			foreach thissock [wwwusers] {
				lappend sockqueue($thissock) "<TABLE CELLPADDING=0 CELLSPACING=0><TR VALIGN=TOP><TD><FONT FACE=FixedSys COLOR=#aa4488>* </FONT></TD>"
				lappend sockqueue($thissock) "<TD><FONT FACE=FixedSys><FONT COLOR=#aa4488>$nick $pubact</FONT></TD></TR></TABLE>"
			}
		}
	}
	return 0
}

proc ircstatus { color arg } {
	return "<TABLE CELLPADDING=0 CELLSPACING=0 WIDTH=100%><TR VALIGN=TOP><TD><FONT FACE=FixedSys COLOR=#$color>*** $arg</FONT></TD></TR></TABLE>"
}

# Other channel specific bindings...
proc sock_join {nick uhost hand chan} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 008800 "$nick \($uhost\) has joined $chan"]
	}
	return 0
}
proc sock_part {nick uhost hand chan} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 008800 "$nick \($uhost\) has left $chan"]
	}
	return 0
}
proc sock_sign {nick uhost hand chan reason} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 000088 "$nick \($uhost\) quits: [htmlclean $reason]"]
	}
	return 0
}
proc sock_kick	{nick uhost hand chan kicked reason} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 008800 "$kicked was kicked by $nick from $chan \([mirc2html [htmlclean $reason]]\)"]
	}
	return 0
}
proc sock_nick {nick uhost hand chan newnick} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 008800 "$nick is now known as $newnick"]
	}
	return 0
}
proc sock_splt {nick uhost hand chan} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus ff0000 "$nick ($uhost) got netsplit in $chan"]
	}
	return 0
}
proc sock_rejn {nick uhost hand chan} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus ff0000 "$nick ($uhost) returned from netsplit in $chan"]
	}
	return 0
}
proc sock_mode {nick uhost hand chan modechange} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 008800 "$nick ($uhost) sets $chan mode: $modechange"]
	}
	return 0
}
proc sock_topc {nick uhost hand chan topic} {
	global sockqueue
	foreach thissock [wwwusers] {
		lappend sockqueue($thissock) [ircstatus 000088 "$nick changes topic in $chan: [htmlclean $topic]"]
	}
	return 0
}

# Starts up server on given port
if {![info exist sockserv]} {
	listen $wwwport script new_idx
	set sockserv 1
}

putlog "www2irc $wwwver: Serving on port $wwwport"
